//
//
// example of incorporation of C++ into python using boost-python
#include <boost/python/module.hpp>
#include <boost/python/def.hpp>
#include <complex>
#include <vector>
#include <boost/python/list.hpp>

// Converts a C++ vector to a python list
template <class T>
boost::python::list toPythonList(std::vector<T> vector) {
  typename std::vector<T>::iterator iter;
  boost::python::list list;
  for (iter = vector.begin(); iter != vector.end(); ++iter) {
    list.append(*iter);
  }
  return list;
}



double calcz(std::complex<double> z, std::complex<double> c, double zabsmax)
{
  long nitmax = 1000;
  long nit = 1;
  while( abs(z) < zabsmax && nit < nitmax)
    {
      z = z * z + c;
      nit += 1;
    }
  double ratio = 255.0  * nit / nitmax;
  return ratio;
}


boost::python::list julialoop(long im_width, long im_height, double xwidth, double yheight, double xmin, double ymin, int nitmax)
{
  int nit = 0;
  double zabsmax = 10.0;
  //complex<double> z;
  std::complex<double> c(-0.1,0.65);
  std::vector<double> julia(im_width * im_height, 0.0);
  for (int ix=0; ix < im_width; ++ix)
    {
      for (int iy=0; iy < im_height; ++iy)
	{
	  nit = 0;
	  // Map pixel position to a point in the complex plane
	  std::complex<double> z((1.0 * ix) / im_width * xwidth + xmin, (1.0 * iy) / im_height * yheight + ymin);
	  // Do the iterations
	  julia[ix*im_width+iy] = calcz(z,c,zabsmax);
	}
    }
  return toPythonList(julia);
}

namespace py = boost::python;




BOOST_PYTHON_MODULE(julia2)
{
  boost::python::def("julialoop",julialoop);
}
