cdef extern from "complex.h":
     double cabs(double complex)

import numpy as np
cimport numpy as np

cdef double calcz(double complex z, double complex c, double zabsmax):
    cdef int nit = 0
    cdef double nitmax = 1000.
    cdef double ratio
    while (z.real*z.real + z.imag*z.imag) < zabsmax and nit < nitmax:
        z = z*z + c
        nit += 1
    ratio = nit / nitmax
    return ratio

def julia_loop(int im_width, int im_height, double xwidth, double yheight, double xmin, double ymin, int nitmax,double complex c):
    #print "Calculate the 2D plane..."
    cdef double zabsmax = 10
    cdef np.ndarray[double,ndim=2] julia = np.empty((im_width,im_height),'double')
    #julia = np.zeros((im_width, im_height))
    cdef double xx = im_width * xwidth
    cdef double yy = im_height * yheight
    cdef int ix, iy
    cdef int nit
    for ix in range(im_width):
        for iy in range(im_height):
            nit = 0
            # Map pixel position to a point in the complex plane
            z = complex((ix/xx + xmin),(iy/yy  + ymin))
            # Do the iterations
            julia[ix][iy] = calcz(z,c,zabsmax)
    return julia

